package de.fhhannover.inform.trust.ifmapj_examples;

import de.fhhannover.inform.trust.ifmapj.IfmapJ;
import de.fhhannover.inform.trust.ifmapj.IfmapJHelper;
import de.fhhannover.inform.trust.ifmapj.binding.IfmapStrings;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.identifier.Identifier;
import de.fhhannover.inform.trust.ifmapj.identifier.Identifiers;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentityType;
import de.fhhannover.inform.trust.ifmapj.messages.MetadataLifetime;
import de.fhhannover.inform.trust.ifmapj.messages.PublishDelete;
import de.fhhannover.inform.trust.ifmapj.messages.PublishRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PublishUpdate;
import de.fhhannover.inform.trust.ifmapj.messages.Requests;
import de.fhhannover.inform.trust.ifmapj.metadata.StandardIfmapMetadataFactory;

public class IdentityTesting {
	
	public void run(String[] args) throws InitializationException {

		System.out.println("====== RUNNING IDENTITY TEST ======");
	
		// Use basic authentication.
		SSRC ssrc = IfmapJ.createSSRC(
				Config.BASIC_AUTH_SERVER_URL,
				Config.BASIC_AUTH_USER,
				Config.BASIC_AUTH_PASSWORD,
				IfmapJHelper.getTrustManagers(
						getClass().getResourceAsStream(Config.TRUST_STORE_PATH),
						Config.TRUST_STORE_PASSWORD));	
		

		// To create simple requests, we need some factories.
		StandardIfmapMetadataFactory mF = IfmapJ.createStandardMetadataFactory();
		
		Identifier id = Identifiers.createIdentity(IdentityType.distinguishedName,
		"CN=Jeff Smith,OU=Sales,DC=Fabrikam,DC=COM");
		
		Identifier ar = Identifiers.createAr("AR0");
		
		
		// Create a PublishUpdate object containing a link. Note: It doesn't
		// really make sense to publish ip-mac metadata between to IP addresses.
		PublishUpdate pu = Requests.createPublishUpdate(ar, id, mF.createAuthAs());
		PublishDelete pd = Requests.createPublishDelete(id, ar, "meta:authenticated-as");
		pd.addNamespaceDeclaration("meta", IfmapStrings.STD_METADATA_NS_URI);
		pu.setLifeTime(MetadataLifetime.forever);
		PublishRequest pr1 = Requests.createPublishReq(pu);
		PublishRequest pr2 = Requests.createPublishReq(pd);

		try {
			// create a new session
			System.out.print("newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			
			// publish the loop
			System.out.print("publish update ... ");
			ssrc.publish(pr1);
			System.out.println("OK");
			ssrc.endSession();
		
			//System.out.println("PRESS RETURN FOR DELETE");
			//System.console().readLine();
			System.out.print("newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			System.out.print("delete ... ");
			ssrc.publish(pr2);
			System.out.println("OK");
			ssrc.endSession();
			
		// catch errors that may occur:
		// - If the server replies with an ErrorResult, a IfmapErrorResult
		//    is thrown.
		// - Other failures, for example due to network outage result in
		//   an IfmapException.
		} catch (IfmapException e) {
			System.err.println(e.getDescription());
		} catch (IfmapErrorResult e) {
			System.err.println(e);
		}
	}

}
